"""
💬 chatspec.create

Contains the `create` module, which is used for creating
chat completions / structured outputs and other LLM related
operations quickly & easily using the `chatspec` package.

This module requires `litellm` and `instructor` to be installed &
can be installed using either:

```bash
pip install litellm instructor
```

or

```bash
pip install chatspec[create]
```
"""

