"""
💬 prompted._create

Contains the `Create` module, which is used for creating
chat completions / structured outputs and other LLM related
operations quickly & easily using the `prompted` package.

This module requires `litellm` and `instructor` to be installed &
can be installed using either:

```bash
pip install litellm instructor
```

or

```bash
pip install prompted[create]
```
"""